Enum kas_core::config::Format

source ·
#[non_exhaustive]
pub enum Format { None, Json, Toml, Yaml, Ron, Unknown, }
Configuration serialisation formats

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


None

Not specified: guess from the path



Json

JavaScript Object Notation



Toml

Tom’s Obvious Minimal Language



Yaml

YAML Ain’t Markup Language



Ron

Rusty Object Notation



Unknown

Error: unable to guess format

Implementations

source

impl Format

source

pub fn guess_from_path(path: &Path) -> Format

Guess format from the path name

This does not open the file.

Potentially fallible: on error, returns Format::Unknown. This may be due to unrecognised file extension or due to the required feature not being enabled.

source

pub fn read_path<T: DeserializeOwned>(self, path: &Path) -> Result<T, Error>

Read from a path

source

pub fn write_path<T: Serialize>( self, path: &Path, value: &T ) -> Result<(), Error>

Write to a path

source

pub fn guess_and_read_path<T: DeserializeOwned>(path: &Path) -> Result<T, Error>

Guess format and load from a path

source

pub fn guess_and_write_path<T: Serialize>( path: &Path, value: &T ) -> Result<(), Error>

Guess format and write to a path