Struct kas_core::draw::SharedState

source ·
pub struct SharedState<DS: DrawSharedImpl> {
    pub draw: DS,
    /* private fields */
}
Shared draw state

A single SharedState instance is shared by all windows and draw contexts. This struct is built over a [DrawSharedImpl] object provided by the shell, which may be accessed directly for a lower-level API (though most methods are available through SharedState directly).

Note: all functionality is implemented through the DrawShared trait to allow usage where the DS type parameter is unknown. Some functionality is also implemented directly to avoid the need for downcasting.

Fields

draw: DS

The shell’s [DrawSharedImpl] object