Enum kas_core::ScrollBarMode
source · pub enum ScrollBarMode {
Fixed,
Auto,
Invisible,
}Expand description
Scroll bar mode
Note that in addition to this mode, bars may be disabled on each axis.
Variants
Fixed
Scroll bars are always shown if enabled.
Auto
Automatically enable/disable scroll bars as required when resized.
This has the side-effect of reserving enough space for scroll bars even when not required.
Invisible
Scroll bars float over content and are only drawn when hovered over by the mouse.