pub enum IsUsed {
Unused,
Used,
}Expand description
Return type of event-handling methods
This type is convertible to/from bool and supports the expected bit-wise
OR operator (a | b, *a |= b).
The type also implements negation with output type bool, thus allowing
if is_used.into() { ... } and if !is_used { ... }. An implementation of
Deref would be preferred, but the trait can only output a reference.
Variants
Unused
Event was unused
Unused events may be used by a parent/ancestor widget or passed to another handler until used.
Used
Event is used, no other result