Enum kas_core::event::Scroll

source ·
pub enum Scroll {
    None,
    Scrolled,
    Offset(Offset),
    Rect(Rect),
}
Request to / notification of scrolling from a child

See: EventCx::set_scroll.

Variants



None

No scrolling



Scrolled

Child has scrolled; no further scrolling needed

External scroll bars use this as a notification to update self.



Offset(Offset)

Pan region by the given offset

This may be returned to scroll the closest scrollable ancestor region. This region should attempt to scroll self by this offset, then, if all the offset was used, return Scroll::Scrolled, otherwise return Scroll::Offset(delta) with the unused offset delta.

With the usual scroll offset conventions, this delta must be subtracted from the scroll offset.



Rect(Rect)

Focus the given rect