pub struct PressStart {
pub source: PressSource,
pub id: Option<Id>,
/* private fields */
}Expand description
Details of press start events
This type dereferences to PressSource.
Fields
source: PressSourceSource of the press
id: Option<Id>Identifier of the widget currently under the press
Implementations
Sourceimpl PressStart
impl PressStart
Sourcepub fn grab(&self, id: Id, mode: GrabMode) -> GrabBuilder
pub fn grab(&self, id: Id, mode: GrabMode) -> GrabBuilder
Grab pan/move/press-end events for widget id
There are three types of grab (GrabMode):
Click: send the correspondingEvent::PressEndonlyGrab(the default): sendEvent::PressMoveandEvent::PressEnd- Pan modes: send
Event::Panon motion. Note: this is most useful when grabbing multiple touch events.
Only a single mouse grab is allowed at any one time; requesting a
second will cancel the first (sending Event::PressEnd with
success: false).
EventState::is_depressed will return true for the grabbing widget.
Call EventCx::set_grab_depress on PressMove to update the
grab’s depress target. (This is done automatically for
GrabMode::Click, and ends automatically when the grab ends.)
This method uses the builder pattern. On completion, Used
is returned. It is expected that the requested press/pan events are all
“used” (Used).
Sourcepub fn grab_click(&self, id: Id) -> GrabBuilder
pub fn grab_click(&self, id: Id) -> GrabBuilder
Convenience wrapper for Self::grab using GrabMode::Click
Sourcepub fn grab_move(&self, id: Id) -> GrabBuilder
pub fn grab_move(&self, id: Id) -> GrabBuilder
Convenience wrapper for Self::grab using GrabMode::Grab