AlignHints

Struct AlignHints 

pub struct AlignHints {
    pub horiz: Option<Align>,
    pub vert: Option<Align>,
}
Partial alignment information provided by the parent

Hints are optional. Widgets are expected to substitute default values where hints are not provided.

The AlignHints::complete method is provided to conveniently apply alignment to a widget within crate::Layout::set_rect:

fn set_rect(&mut self, _: &mut SizeCx, rect: Rect, align: AlignHints) {
    let pref_size = Size(30, 20); // usually size comes from SizeCx
    let rect = align
        .complete(Align::Stretch, Align::Center)
        .aligned_rect(pref_size, rect);
    self.core.set_rect(rect);
}

Fields

horiz: Option<Align>vert: Option<Align>

Implementations

Source

impl AlignHints

pub const NONE: AlignHints

No hints

pub const TOP: AlignHints

Top, no horizontal hint

pub const BOTTOM: AlignHints

Bottom, no horizontal hint

pub const LEFT: AlignHints

Left, no vertical hint

pub const RIGHT: AlignHints

Right, no vertical hint

pub const TOP_LEFT: AlignHints

Top, left

pub const TOP_RIGHT: AlignHints

Top, right

pub const BOTTOM_LEFT: AlignHints

Bottom, left

pub const BOTTOM_RIGHT: AlignHints

Bottom, right

pub const CENTER: AlignHints

Center on both axes

pub const TOP_CENTER: AlignHints

Top, center

pub const BOTTOM_CENTER: AlignHints

Bottom, center

pub const CENTER_LEFT: AlignHints

Center, left

pub const CENTER_RIGHT: AlignHints

Center, right

pub const HORIZ_CENTER: AlignHints

Center horizontally

pub const VERT_CENTER: AlignHints

Center vertically

pub const STRETCH: AlignHints

Stretch on both axes

pub const fn new(horiz: Option<Align>, vert: Option<Align>) -> Self

Construct with optional horiz. and vert. alignment

pub fn extract(self, dir: impl Directional) -> Option<Align>

Take horizontal/vertical component

pub fn set_component<D: Directional>(&mut self, dir: D, align: Option<Align>)

Set one component of self, based on a direction

pub fn combine(self, rhs: AlignHints) -> Self

Combine two hints (first takes priority)

pub fn unwrap_or(self, horiz: Align, vert: Align) -> (Align, Align)

Unwrap type’s alignments or substitute parameters

pub fn complete(&self, horiz: Align, vert: Align) -> AlignPair

Complete, using specified alignments as defaults

pub fn complete_default(&self) -> AlignPair

Complete, using Align::Default as defaults

pub fn complete_center(&self) -> AlignPair

Complete, using Align::Center as defaults