ReadMessage

Trait ReadMessage 

pub trait ReadMessage {
    // Required methods
    fn pop_erased(&mut self) -> Option<Erased>;
    fn try_pop<M: Debug + 'static>(&mut self) -> Option<M>;
    fn try_peek<M: Debug + 'static>(&self) -> Option<&M>;
    fn peek_debug(&self) -> Option<&dyn Debug>;
}
This trait allows peeking and popping messages from the stack

Required Methods

fn pop_erased(&mut self) -> Option<Erased>

Pop a type-erased message from the stack, if non-empty

fn try_pop<M: Debug + 'static>(&mut self) -> Option<M>

Try popping the last message from the stack with the given type

fn try_peek<M: Debug + 'static>(&self) -> Option<&M>

Try observing the last message on the stack without popping

fn peek_debug(&self) -> Option<&dyn Debug>

Debug the last message on the stack, if any

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors