Struct kas_core::shell::Shell

source ·
pub struct Shell<Data: AppData, G: GraphicalShell, T: Theme<G::Shared>> { /* private fields */ }
The KAS shell

The “shell” is the layer over widgets, windows, events and graphics.

Constructing with Shell::new or Shell::new_custom reads configuration (depending on passed options or environment variables) and initialises the font database. Note that this database is a global singleton and some widgets and other library code may expect fonts to have been initialised first.

Implementations

source

impl<Data: AppData, G, T> Shell<Data, G, T>where G: GraphicalShell + Default, T: Theme<G::Shared> + 'static, T::Window: Window,

source

pub fn new(data: Data, theme: T) -> Result<Self>

Construct a new instance with default options.

All user interfaces are expected to provide data: Data: widget data shared across all windows. If not required this may be ().

Environment variables may affect option selection; see documentation of Options::from_env. KAS config is provided by Options::read_config.

source

impl<Data: AppData, G: GraphicalShell, T> Shell<Data, G, T>where T: Theme<G::Shared> + 'static, T::Window: Window,

source

pub fn new_custom( data: Data, graphical_shell: impl Into<G>, theme: T, options: Options ) -> Result<Self>

Construct an instance with custom options

The Options parameter allows direct specification of shell options; usually, these are provided by Options::from_env.

KAS config is provided by Options::read_config and theme is configured through Options::init_theme_config.

source

pub fn new_custom_config( data: Data, graphical_shell: impl Into<G>, theme: T, options: Options, config: Rc<RefCell<Config>> ) -> Result<Self>

Construct an instance with custom options and config

This is like Shell::new_custom, but allows KAS config to be specified directly, instead of loading via Options::read_config.

Unlike other the constructors, this method does not configure the theme. The user should call Options::init_theme_config before this method.

source

pub fn draw_shared(&mut self) -> &mut dyn DrawShared

Access shared draw state

source

pub fn theme(&self) -> &T

Access the theme by ref

source

pub fn theme_mut(&mut self) -> &mut T

Access the theme by ref mut

source

pub fn add(&mut self, window: Window<Data>) -> WindowId

Assume ownership of and display a window

source

pub fn with(self, window: Window<Data>) -> Self

Assume ownership of and display a window, inline

source

pub fn create_proxy(&self) -> Proxy

Create a proxy which can be used to update the UI from another thread

source

pub fn run(self) -> Result<()>

Run the main loop.