Module kas_core::theme

source ·
Themes

Widgets expect the theme to provide an implementation of SizeCx and of DrawCx.

Constructing a shell requires a Theme. Two implementations are provided here: SimpleTheme and FlatTheme. An adapter, MultiTheme, is also provided.

Structs

Enums

Traits

  • A theme provides widget sizing and drawing implementations.
  • Requirements on theme config (with config feature)
  • Interface through which a theme can be adjusted at run-time
  • As Theme, but without associated types
  • Per-window storage for the theme

Type Aliases