pub enum Ime<'a> {
Enabled,
Disabled,
Preedit {
text: &'a str,
cursor: Option<(usize, usize)>,
},
Commit {
text: &'a str,
},
DeleteSurrounding {
before_bytes: usize,
after_bytes: usize,
},
}Expand description
Input Method Editor events
This enum describes input method events.
This enum closely follows the specification of winit::event::Ime,
hence its documentation may prove useful.
Variants
Enabled
Notifies when the IME was enabled.
This may mean, for example, that a virtual keyboard is now active with focus on the recipient. This event does not mean that physical keyboard input is disabled (or that it should be ignored).
The widget should call EventState::set_ime_cursor_area immediately
and each time the area changes (relative to the widget’s coordinate
space), until Ime::Disabled is received. Failure to do so will
result in the widget’s entire rect being used as the IME cursor area.
Disabled
Notifies when the IME was disabled.
Any pending (uncommitted) pre-edit should be cleared (cancelled).
Preedit
Notifies when a new composing text should be set at the cursor position.
When text.is_empty() the pre-edit was cleared. Synthetic events with
empty text and cursor: None are generated immediately before an
Ime::Commit event.
cursor represents the selection-start and cursor indices within
text. If cursor.is_none() then no cursor should be shown.
Commit
Notifies when text should be inserted into the editor widget.
Right before this event winit will send empty Self::Preedit event.
DeleteSurrounding
Delete text surrounding the cursor or selection.
This event does not affect the pre-edit string.