Themes
Widgets expect the theme to provide an implementation of SizeCx and of
DrawCx.
Constructing a shell requires a Theme. Two implementations are provided
here: SimpleTheme and FlatTheme.
An adapter, MultiTheme, is also provided.
Structs
- Provides standard theme colours
- Event handling configuration
- Draw interface
- A theme with flat (unshaded) rendering
- Input and highlighting state of a widget
- Wrapper around multiple themes, supporting run-time switching
- Builder for
MultiTheme - Font raster settings
- A simple theme
- Size and scale interface
Enums
- Optional background colour
- Various features which may be sized and drawn
- Style of a frame
- Margin size
- Style of marks
- An optionally-owning (boxed) reference
- Selection style hint
- Class of text drawn
Traits
- A theme provides widget sizing and drawing implementations.
- Requirements on theme config (with
configfeature) - Interface through which a theme can be adjusted at run-time
- As
Theme, but without associated types - Per-window storage for the theme